#!/bin/bash
###########################################
# Software Requirement:                   #
#   bash x11-utils awk wmctrl bc          #
# License:                                #
#   This script is licensed under GPL 3.0 #
# Version:                                #
#   0.3                                   #
# Author:                                 #
#   Alan CHENG <coolcd05a@gmail.com>      #
###########################################
## Custimization
# Path to essential commands
WMCTRL=`which wmctrl`
# DISTANCE_HORI is the distance to move horizontally
DISTANCE_HORI=16
# DISTANCE_VERT is the distance to move vertically
DISTANCE_VERT=16
# DISTANCE_SLASH is the distance to move when using MOVE_UR, MOVE_UL, MOVE_DR, MOVE_DL
DISTANCE_SLASH=16
# DELAY is the time delay between each pixel movement
# PAUSE is the pause between MOVE_L, MOVE_D, MOVE_R, MOVE_U...
DELAY_HORI=0.02
DELAY_VERT=0.02
DELAY_SLASH=0.02
PAUSE=0.08
# Screen Width and Height
RESOLUTION=`wmctrl -d|awk '/\*/{ print $4}'`
SCR_W=`echo ${RESOLUTION} |awk -F x '{ print $1 }'`
SCR_H=`echo ${RESOLUTION} |awk -F x '{ print $2 }'`
# DEP_X and DEP_Y atr Allowed Pixels outside the screen
# Pixels outside top and left edges don't work,
# this may be a bug of wmctrl.
DEP_X=24
DEP_Y=24

## Functions
#  POS_WINDOWS() determins the position of windows
POS_WINDOWS () {
		#L=(`${WMCTRL} -Gl`);
		#echo '$L'=$L;
		CUR_X=`xwininfo -id $w|awk '$0 ~ /Absolute upper-left X/{ print $4; }'`
		CUR_Y=`xwininfo -id $w|awk '$0 ~ /Absolute upper-left Y/{ print $4; }'`
		CUR_W=`xwininfo -id $w|awk '$0 ~ /Width/{ print $2; }'`
		CUR_H=`xwininfo -id $w|awk '$0 ~ /Height/{ print $2; }'`
		#echo '$CUR_Y'=${CUR_Y};
		#echo '$CUR_X'=${CUR_X};
}
#  SHOW_WINDOWS() shows you the windows you can swing
SHOW_WINDOWS () {
	echo "#######################################################"
	${WMCTRL} -l|awk '{ print $1"  "$4" "$5" "$6" "$7  }'
	echo "#######################################################"
	select w in `${WMCTRL} -l|awk '{ print $1 }'`
	do W="$w" ;
		echo You have selected $w to swing ;
		POS_WINDOWS
		return;
	done
}

# MOVE_R, MOVE_D, MOVE_L, MOVE_U move the window you selected
MOVE_R () {
	for (( x=0 ; x<= ${DISTANCE_HORI}; x++ )) ;
		do let CUR_X=${CUR_X}+1;
		if (( "${CUR_X}"+"${CUR_W}" >= "${SCR_W}"+"${DEP_X}" )) ;then
			return;
		else
			#echo '${CUR_X}'=${CUR_X}
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			POS_WINDOWS
			sleep ${DELAY_HORI};
		fi
		done;
}

MOVE_D () {
	for (( y=0 ; y<= ${DISTANCE_VERT}; y++ )) ;
		do let CUR_Y=${CUR_Y}+1;
		if (( "${CUR_Y}"+"${CUR_H}" >= "${SCR_H}"+"${DEP_Y}" )) ;then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			POS_WINDOWS
			sleep ${DELAY_VERT};
		fi
		done;
}

MOVE_L () {
	for (( x=0 ; x <= ${DISTANCE_HORI}; x++ )) ;
		do let CUR_X=${CUR_X}-1;
		if (( "${CUR_X}" <= 0-"${DEP_X}" )) ;then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			POS_WINDOWS
			sleep ${DELAY_HORI};
		fi
		done;
}

MOVE_U () {
	for (( y=0 ; y<= ${DISTANCE_VERT}; y++ )) ;
		do let CUR_Y=${CUR_Y}-1;
		if (( "${CUR_Y}" <= 0-"${DEP_Y}" )) ;then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			POS_WINDOWS
			sleep ${DELAY_VERT};
		fi
		done;
}

MOVE_UR () {
	for (( y=0 ; y<= `echo "${DISTANCE_SLASH}/sqrt(2)"|bc` ; y++ )) ;
		do let CUR_Y=${CUR_Y}-1;
		let CUR_X=${CUR_X}+1;
		if (( "${CUR_Y}" <= 0-"${DEP_Y}" )) || (( "${CUR_X}" + "${CUR_W}" >= "${SCR_W}"+"${DEP_X}" )); then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			POS_WINDOWS
			sleep ${DELAY_SLASH};
		fi
		done;
}

MOVE_UL () {
	for (( y=0 ; y<= `echo "${DISTANCE_SLASH}/sqrt(2)"|bc` ; y++ )) ;
		do let CUR_Y=${CUR_Y}-1;
		let CUR_X=${CUR_X}-1;
		if (( "${CUR_Y}" <= 0-"${DEP_Y}" )) || (( "${CUR_X}" <= 0-"${DEP_X}" )); then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			POS_WINDOWS
			sleep ${DELAY_SLASH};
		fi
		done;
}

MOVE_DR () {
	for (( y=0 ; y<= `echo "${DISTANCE_SLASH}/sqrt(2)"|bc` ; y++ )) ;
		do let CUR_Y=${CUR_Y}+1;
		let CUR_X=${CUR_X}+1;
		if (( "${CUR_Y}" + "${CUR_H}" >= "${SCR_H}"+"${DEP_Y}" )) || (( "${CUR_X}" + "${CUR_W}" >= "${SCR_W}" + "${DEP_X}" )); then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			POS_WINDOWS
			sleep ${DELAY_SLASH};
		fi
		done;
}

MOVE_DL () {
	for (( y=0 ; y<= `echo "${DISTANCE_SLASH}/sqrt(2)"|bc` ; y++ )) ;
		do let CUR_Y=${CUR_Y}+1;
		let CUR_X=${CUR_X}-1;
		if (( "$CUR_Y" + "$CUR_H" >= "${SCR_H}" + "${DEP_Y}" )) || (( "$CUR_X" <= 0 - "${DEP_X}")); then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			POS_WINDOWS
			sleep ${DELAY_SLASH};
		fi
		done;
}

SHOW_WINDOWS
# You can change the order of movement as you like
while true
do
	MOVE_L;
	sleep ${PAUSE}
	MOVE_DL;
	sleep ${PAUSE}
	MOVE_D;
	sleep ${PAUSE}
	MOVE_DR;
	sleep ${PAUSE}
	MOVE_R;
	sleep ${PAUSE}
        MOVE_UR;
	sleep ${PAUSE}
	MOVE_U;
	sleep ${PAUSE}
	MOVE_UL;
	sleep ${PAUSE}
done
